@@ -443,94 +443,61 @@ class LupusecAPI:
         # Get covers from cache or update from Lupusec System
         # ToDo: update frequency shall not be hard-coded -> transfer to constant
         if self._cacheCovers is None or timeNow - self._cacheStampP > 2.0:
-            _LOGGER.debug("...covers need update from Lupusec System.")
+            _LOGGER.debug("get_covers_async(): covers need update from Lupusec System.")
             self._cacheStampP = timeNow
 
-            async with aiohttp.ClientSession(auth=self._auth) as session:
-                _LOGGER.debug("auth.encode: %s", self._auth.encode())  
+            # Make API-Call to update
+            _LOGGER.debug("get_covers_async(): make API-call with get_devices_async() to Lupusec System.")            
+            await LupusecAPI.get_devices_async(self)
 
-                # Get all devices and filter for covers
-                _LOGGER.debug("__init__.py.get_covers_async(): REQUEST=%s", CONST.DEVICE_LIST_REQUEST)
-                get_covers_response = await LupusecAPI._async_api_call(self._ip_address, session, CONST.DEVICE_LIST_REQUEST)
-                _LOGGER.debug("_async_api_call(): done. check response...")
-                # Retreive Device Liste from Response
-                if CONST.DEVICE_LIST_HEADER in get_covers_response:
-                    device_content = get_covers_response[CONST.DEVICE_LIST_HEADER]
-                    print("Number of devices=", len(device_content))    
-                    if (len(device_content) != 0):             
-                        covers = []
-                        for device in device_content:
-                            _LOGGER.debug("Device found: sid=%s, name=%s, type=%s, status=%s", 
-                                device["sid"], device["name"], device["type"], device["status_ex"] )
-                            if (device["type"] in CONST.TYPES_COVER):    
-                                covers.append(device)
-                                _LOGGER.debug("device is shutter...added.")
-                            else :
-                                _LOGGER.debug("device is no shutter...skipping.")
-                        self._cacheCovers = covers
-                    else : 
-                        _LOGGER.info("ERROR: get_covers_async(): no covers found.")
-                else :
-                    _LOGGER.info("ERROR: get_covers_async(): no covers found.")
+            _LOGGER.debug("get_covers_async(): API-call finished. check response...")
+            # Check response 
+            if (len(self._cacheCovers) != 0):
+                _LOGGER.debug("...number of covers=%s", len(self._cacheCovers))
+            else:
+                _LOGGER.info("ERROR: get_covers_async(): no covers found.")
 
         _LOGGER.debug("__init__.py.get_covers_async() finished.") 
 
+        _LOGGER.debug("get_covers_async(): checking cacheCovers...")
         if (len(self._cacheCovers) != 0):
-            _LOGGER.debug("Number of covers=%s", len(self._cacheCovers)) 
-
+            _LOGGER.debug("...number of covers=%s", len(self._cacheCovers))
+        else:
+             _LOGGER.info("ERROR: get_covers_async(): no covers found.")           
+        # return dictionary with covers
         return self._cacheCovers
 
 
-    async def get_binary_sensors(self) -> Dict:
+    async def get_binary_sensors_async(self) -> Dict:
         """Async method to get BinarySensors."""
-        _LOGGER.debug("__init__.py.get_binary_sensors() called: ")
+        _LOGGER.debug("__init__.py.get_binary_sensors_async() called: ")
         timeNow = time.time()
         
-        # Get switches from cache or update from Lupusec System
+        # Get Binary Sensors from cache or update from Lupusec System
         # ToDo: update frequency shall not be hard-coded -> transfer to constant
         if self._cacheBinarySensors is None or timeNow - self._cacheStampP > 2.0:
-            _LOGGER.debug("...BinarySensors need update from Lupusec System.")
+            _LOGGER.debug("get_binary_sensors_async(): BinarySensors need update from Lupusec System.")
             self._cacheStampP = timeNow
 
-            async with aiohttp.ClientSession(auth=self._auth) as session:
-                _LOGGER.debug("auth.encode: %s", self._auth.encode())  
+             # Make API-Call to update
+            _LOGGER.debug("get_binary_sensors_async(): make API-call with get_devices_async() to Lupusec System.")            
+            await LupusecAPI.get_devices_async(self)
 
-                # Get all devices and filter for switches
-                _LOGGER.debug("__init__.py.async_get_devices(): REQUEST=%s", CONST.DEVICE_LIST_REQUEST)
-                get_bin_sensors_response = await LupusecAPI._async_api_call(self._ip_address, session, CONST.DEVICE_LIST_REQUEST)
-                _LOGGER.debug("_async_api_call(): done. check response...")
-                # Retreive Device Liste from Response
-                if CONST.DEVICE_LIST_HEADER in get_bin_sensors_response:
-                    device_content = get_bin_sensors_response[CONST.DEVICE_LIST_HEADER]
-                    print("Number of devices=", len(device_content))    
-                    if (len(device_content) != 0):             
-                        binary_sensors = []
-                        for device in device_content:
-                            _LOGGER.debug("BinarySensor found: sid=%s, name=%s, type=%s, status=%s", 
-                                device["sid"], device["name"], device["type"], device["status_ex"] )
-                            if (device["type"] in CONST.TYPES_BIN_SENSOR):    
-                                binary_sensors.append(device)
-                                _LOGGER.debug("device is BinarySensor...added.")
-                            else :
-                                _LOGGER.debug("device is no BinarySensor...skipping.")
-                        self._cacheBinarySensors = binary_sensors
-                    else : 
-                        _LOGGER.info("ERROR: get_binary_sensors(): no BinarySensors found.")
-                else :
-                    _LOGGER.info("ERROR: get_binary_sensors(): no BinarySensors found.")
+            _LOGGER.debug("get_binary_sensors_async(): API-call finished. check response...")
+            # Check response 
+            if (len(self._cacheBinarySensors) != 0):
+                _LOGGER.debug("...number of binary sensors=%s", len(self._cacheBinarySensors))
+            else:
+                _LOGGER.info("ERROR: get_binary_sensors_async(): no binary sensors  found.")
 
-        _LOGGER.debug("__init__.py.get_binary_sensors() finished.") 
+        _LOGGER.debug("__init__.py.get_binary_sensors_async() finished.") 
 
+        _LOGGER.debug("get_binary_sensors_async(): checking cacheBinarySensors...")
         if (len(self._cacheBinarySensors) != 0):
-            _LOGGER.debug("Number of BinarySensors=%s", len(self._cacheBinarySensors)) 
-            print("Number of BinarySensors=", len(self._cacheBinarySensors))            
-            for sensor in self._cacheBinarySensors:
-                print("sid: ", sensor["sid"], 
-                    ", name: ", sensor["name"], 
-                    ", type: ", sensor["type"], 
-                    ", area: ", sensor["area"],
-                    ", zone: ", sensor["zone"],                     
-                    ", status: ", sensor["status_ex"])
+            _LOGGER.debug("...number of binary sensors=%s", len(self._cacheBinarySensors))
+        else:
+             _LOGGER.info("ERROR: get_binary_sensors_async(): no binary sensors found.")           
+        # return dictionary with binary sensors
         return self._cacheBinarySensors
 
 
@@ -597,130 +564,74 @@ class LupusecAPI:
 
 
     async def get_switch_async(self, device_id) -> Dict:
-        """Async method to get a single switch device."""
+        """ Async method to get a single switch device. """
+        """ device will be taken from cache or updated from API-Call """
         _LOGGER.debug("__init__.py.get_switch_async() called for device_id=%s", device_id)
-        timeNow = time.time()
         
         # Get device from cache or update from Lupusec System
-        # ToDo: update frequency shall not be hard-coded -> transfer to constant
-        if self._cacheSwitches is None or timeNow - self._cacheStampP > 10.0:
-            # time elapsed, devices need update from Lupusec System.
-            if self._cacheSwitches is None:
-                _LOGGER.debug("__init__.py.get_switch_async(): cacheSwitchess is none, update from Lupusec System started.")
-            if timeNow - self._cacheStampP > 2.0:
-                _LOGGER.debug("__init__.py.get_switch_async(): time elapsed, update from Lupusec System started.")    
-            self._cacheStampP = timeNow
+        get_devices = await LupusecAPI.get_switches_async(self)
 
-            async with aiohttp.ClientSession(auth=self._auth) as session:
-                _LOGGER.debug("auth.encode: %s", self._auth.encode())  
-
-                # Get all devices from LupusecAPI
-                _LOGGER.debug("__init__.py.get_switch_async(): REQUEST=%s", CONST.DEVICE_LIST_REQUEST)
-                get_bin_switches_response = await LupusecAPI._async_api_call(self._ip_address, session, CONST.DEVICE_LIST_REQUEST)
-                _LOGGER.debug("_async_api_call(): done. check response...")
-                # Retreive Device Liste from Response
-                if CONST.DEVICE_LIST_HEADER in get_bin_switches_response:
-                    device_content = get_bin_switches_response[CONST.DEVICE_LIST_HEADER]
-                    _LOGGER.debug("Number of devices found: %s", len(device_content))    
-                    if (len(device_content) != 0):             
-                        switches = []
-                        for device in device_content:
-                            if (device["type"] in CONST.TYPES_SWITCH):    
-                                switches.append(device)
-                                _LOGGER.debug("Switch found: sid=%s, name=%s, type=%s, status=%s", 
-                                    device["sid"], device["name"], device["type"], device["status_ex"] )
-                                _LOGGER.debug("sid=%s, python_class=%s", 
-                                    device["sid"], type(device["sid"]))                                    
-                            else :
-                                _LOGGER.debug("device is no Switch...skipping.")
-                        self._cacheSwitches = switches
-                        _LOGGER.debug("get_switch_async(): No of updated switches: %s", len(self._cacheSwitches))
-                    else : 
-                        _LOGGER.info("ERROR: get_switch_async(): no Switch found.")
-                else :
-                    _LOGGER.info("ERROR: get_switch_async(): no Switch found.")
-            _LOGGER.debug("__init__.py.get_switch_async(): update from LupusecAPI finished.") 
-
-
-        # time not elapsed, Devices do not need update from Lupusec System...using Cache
         _LOGGER.debug("__init__.py.get_switch_async(): using Cache to provide Update on Switch.")
-        if (len(self._cacheSwitches) != 0):  
-            _LOGGER.debug("__init__.py.get_switch_async(): Number of Switches=%s", len(self._cacheSwitches))             
-            for switch in self._cacheSwitches:
-                #_LOGGER.debug("__init__.py.get_switch_async(): device with sid=%s, name=%s", sensor["sid"], sensor["name"])
-                if (switch["sid"] == device_id):
-                    _LOGGER.debug("__init__.py.get_switch_async(): matching device with sid=%s found: status=%s", switch["sid"], switch["status_ex"])
-                    return switch    
-            _LOGGER.error("__init__.py.get_cover_async(): %s Switches checked, but no device found with sid=%s", len(self._cacheSwitches), device_id)
+        if (len(get_devices) != 0):  
+            _LOGGER.debug("__init__.py.get_switch_async(): Number of Switches=%s", len(get_devices))             
+            for device in get_devices:
+                #_LOGGER.debug("__init__.py.get_switch_async(): device with sid=%s, name=%s", device["sid"], device["name"])
+                if (device["sid"] == device_id):
+                    _LOGGER.debug("__init__.py.get_switch_async(): matching device with sid=%s found: status=%s", device["sid"], device["name"])
+                    return device    
+            _LOGGER.error("__init__.py.get_switch_async(): %s Switches checked, but no device found with sid=%s", len(get_devices), device_id)
             return {}
         else :
-            _LOGGER.debug("ERROR: length of self._cacheSwitches = 0") 
+            _LOGGER.debug("ERROR: length of get_devices = 0") 
             return {}
 
 
+
     async def get_cover_async(self, device_id) -> Dict:
         """Async method to get a single cover device."""
         _LOGGER.debug("__init__.py.get_cover_async() called for device_id=%s", device_id)
-        timeNow = time.time()
-        
+
         # Get device from cache or update from Lupusec System
-        # ToDo: update frequency shall not be hard-coded -> transfer to constant
-        if self._cacheCovers is None or timeNow - self._cacheStampP > 10.0:
-            # time elapsed, devices need update from Lupusec System.
-            if self._cacheCovers is None:
-                _LOGGER.debug("__init__.py.get_covers_async(): cacheCovers is none, update from Lupusec System started.")
-            if timeNow - self._cacheStampP > 2.0:
-                _LOGGER.debug("__init__.py.get_covers_async(): time elapsed, update from Lupusec System started.")    
-            self._cacheStampP = timeNow
+        get_devices = await LupusecAPI.get_covers_async(self)
 
-            async with aiohttp.ClientSession(auth=self._auth) as session:
-                _LOGGER.debug("auth.encode: %s", self._auth.encode())  
-
-                # Get all devices from LupusecAPI
-                _LOGGER.debug("__init__.py.get_cover_async(): REQUEST=%s", CONST.DEVICE_LIST_REQUEST)
-                get_bin_covers_response = await LupusecAPI._async_api_call(self._ip_address, session, CONST.DEVICE_LIST_REQUEST)
-                _LOGGER.debug("_async_api_call(): done. check response...")
-                # Retreive Device Liste from Response
-                if CONST.DEVICE_LIST_HEADER in get_bin_covers_response:
-                    device_content = get_bin_covers_response[CONST.DEVICE_LIST_HEADER]
-                    _LOGGER.debug("Number of devices found: %s", len(device_content))    
-                    if (len(device_content) != 0):             
-                        covers = []
-                        for device in device_content:
-                            if (device["type"] in CONST.TYPES_COVER):    
-                                covers.append(device)
-                                _LOGGER.debug("Cover found: sid=%s, name=%s, type=%s, status=%s, onOff=%s, level=%s", 
-                                    device["sid"], device["name"], device["type"], device["status_ex"], device["onOff"], device["level"] )
-                                _LOGGER.debug("sid=%s, python_class=%s", 
-                                    device["sid"], type(device["sid"]))                                    
-                            else :
-                                _LOGGER.debug("device is no Cover...skipping.")
-                        self._cacheCovers = covers
-                        _LOGGER.debug("get_cover_async(): No of updated Covers: %s", len(self._cacheCovers))
-                    else : 
-                        _LOGGER.info("ERROR: get_cover_async(): no Cover found.")
-                else :
-                    _LOGGER.info("ERROR: get_cover_async(): no Cover found.")
-            _LOGGER.debug("__init__.py.get_cover_async(): update from LupusecAPI finished.") 
-
-
-
-        # time not elapsed, Devices do not need update from Lupusec System...using Cache
         _LOGGER.debug("__init__.py.get_cover_async(): using Cache to provide Update on Cover.")
-        if (len(self._cacheCovers) != 0):  
-            _LOGGER.debug("__init__.py.get_cover_async(): Number of Covers=%s", len(self._cacheCovers))             
-            for cover in self._cacheCovers:
-                #_LOGGER.debug("__init__.py.get_cover_async(): device with sid=%s, name=%s", sensor["sid"], sensor["name"])
-                if (cover["sid"] == device_id):
-                    _LOGGER.debug("__init__.py.get_cover_async(): matching device with sid=%s found: status=%s", cover["sid"], cover["status_ex"])
-                    return cover    
-            _LOGGER.error("__init__.py.get_cover_async(): %s Covers checked, but no device found with sid=%s", len(self._cacheCovers), device_id)
+        if (len(get_devices) != 0):  
+            _LOGGER.debug("__init__.py.get_cover_async(): Number of Covers=%s", len(get_devices))             
+            for device in get_devices:
+                #_LOGGER.debug("__init__.py.get_cover_async(): device with sid=%s, name=%s", device["sid"], device["name"])
+                if (device["sid"] == device_id):
+                    _LOGGER.debug("__init__.py.get_cover_async(): matching device with sid=%s found: status=%s", device["sid"], device["name"])
+                    return device    
+            _LOGGER.error("__init__.py.get_cover_async(): %s Covers checked, but no device found with sid=%s", len(get_devices), device_id)
             return {}
         else :
-            _LOGGER.debug("ERROR: length of self._cacheCovers = 0") 
+            _LOGGER.debug("ERROR: length of get_devices = 0") 
             return {}
 
 
+    async def get_binary_sensor_async(self, device_id) -> Dict:
+        """Async method to get a single binary_sensor device."""
+        _LOGGER.debug("__init__.py.get_binary_sensor_async() called for device_id=%s", device_id)
+
+        # Get device from cache or update from Lupusec System
+        get_devices = await LupusecAPI.get_binary_sensor_async(self)
+
+        _LOGGER.debug("__init__.py.get_binary_sensor_async(): using Cache to provide Update on Binary Sensor.")
+        if (len(get_devices) != 0):  
+            _LOGGER.debug("__init__.py.get_binary_sensor_async(): Number of Binary Sensors=%s", len(get_devices))             
+            for device in get_devices:
+                #_LOGGER.debug("__init__.py.get_binary_sensor_async(): device with sid=%s, name=%s", device["sid"], device["name"])
+                if (device["sid"] == device_id):
+                    _LOGGER.debug("__init__.py.get_binary_sensor_async(): matching device with sid=%s found: status=%s", device["sid"], device["name"])
+                    return device    
+            _LOGGER.error("__init__.py.get_binary_sensor_async(): %s Binary Sensors checked, but no device found with sid=%s", len(get_devices), device_id)
+            return {}
+        else :
+            _LOGGER.debug("ERROR: length of get_devices = 0") 
+            return {}
+
+
+
     def get_sensors(self):
         _LOGGER.debug("get_sensors() called:")
         stamp_now = time.time()
